#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include "audio.h"

#define	MIXERDEV	"/dev/mixer"
#define AUDIODEV 	"/dev/dsp"

extern int debug;

int sample_rate;
int sample_time;
int sample_size;
int buffer_size;
int volume;
int stereo;

int mixer,audio;

unsigned char *audio_buffer;

int InitAudio() {
	int bytes, stereo=0, sample_size=8;

	if ((mixer=open(MIXERDEV,O_RDONLY,0))==-1) {
    		perror(MIXERDEV); return -1;
	}
	ioctl(mixer,SOUND_MIXER_WRITE_MIC,&volume);
	close(mixer);

  	if ((audio=open(AUDIODEV,O_RDONLY,0))==-1) {
		perror(AUDIODEV); return -1;
	}
	ioctl(audio, SNDCTL_DSP_GETBLKSIZE,&buffer_size);
	ioctl(audio, SNDCTL_DSP_SYNC,NULL);
	ioctl(audio, SNDCTL_DSP_SAMPLESIZE,&sample_size);
        ioctl(audio, SNDCTL_DSP_STEREO,&stereo);
  	ioctl(audio, SNDCTL_DSP_SPEED,&sample_rate);

	bytes=sample_time*sample_rate*(sample_size/8)*(stereo+1)/10;
	audio_buffer=calloc(bytes,sizeof(unsigned char));
	if (audio_buffer==NULL) {
		perror("audio_buffer"); return -1;
	}

	if (debug) {
		printf("Volume      : %d\n",volume);
		printf("Stereo      : %d\n",stereo);
		printf("Sample rate : %d Hz\n",sample_rate);
		printf("Sample time : %f sec\n",(double)sample_time/10.0);
		printf("Buffer size : %d bytes\n",buffer_size);
		printf("Sample size : %d bytes\n",sample_size);
		printf("Byte/sample : %d\n",bytes);
	}

	return bytes;
}


void CloseAudio() { close(audio); }


int SampleAudio(int bytes) {
	int count, sofar=0, i=0;

	while (sofar<bytes) {
		if ((bytes-sofar)<=buffer_size)
			count=(bytes-sofar);
		else count=buffer_size;

		if ((i=read(audio,audio_buffer+sofar,count))==-1) { 
			perror("audio read"); exit(1); 
		}
		sofar+=i;
#ifdef DEBUG
		printf("bytes sofar : %d\n",sofar);
#endif
	}

	return sofar;
}


void FlushAudio() { ioctl(audio,SNDCTL_DSP_SYNC,0); }

