#include <gnome.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#include "audio.h"
#include "calc.h"
#include "note.h"
#include "interface.h"

#define DEFAULT_INTERVAL 2500

GtkWidget *app, *dialog;
guint timer;
extern int interval = 0;
int n_times = 1;
int N; /* I'm goin a little heavy on the globals ... call it laziness :) */

static GnomeUIInfo tuner_menu[]= {
	/*{
		GNOME_APP_UI_ITEM,
		N_("Preferences"), N_("Set up basic elements of this program"),
		pref_cb, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF,
		0, (GdkModifierType)0, NULL
	},*/
	GNOMEUIINFO_SEPARATOR,
	{ 
		GNOME_APP_UI_ITEM,
		N_("Exit"), N_("Exit Gnome Guitar Tuner"),
		quit_cb, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_QUIT,
		'Q', GDK_CONTROL_MASK, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu[]=
{
	GNOMEUIINFO_HELP("gtune"),
	GNOMEUIINFO_SEPARATOR,
	{
		GNOME_APP_UI_ITEM,
		N_("About..."), N_("About this Gnome Guitar Tuner"),
		about_cb, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT,
		0, (GdkModifierType)0, NULL
	}, 
	GNOMEUIINFO_END
}; 

static GnomeUIInfo main_menu[]= 
{
	GNOMEUIINFO_SUBTREE(N_("Tuner"), tuner_menu),
	GNOMEUIINFO_SUBTREE(N_("Help"), help_menu),
	GNOMEUIINFO_END
};

/* this is a lainblaitant copy of what glade gives you but oh well, it's useful isn't it! */
GtkWidget* 
get_widget(GtkWidget *widget, gchar *widget_name)
{
  	GtkWidget *found_widget;

  	if (widget->parent)
    		widget = gtk_widget_get_toplevel (widget);
  	found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
                                                   widget_name);
  	if (!found_widget)
    		g_warning ("Widget not found: %s", widget_name);
  	return found_widget;
}

void progress_cb()
{
	static gfloat pvalue;
	FILE *f;
	GtkWidget *w, *muck;
	extern int sample_rate, sample_time, volume;
	int context_id;
	
	if(n_times == 1) { pvalue = 0; n_times = 0; }
	w = get_widget(dialog, "progressbar");
		
	if(pvalue >= 1) 
	{
		muck = get_widget(app, "statusbar");
		context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(muck), "Statusbar example");
		gtk_timeout_remove(timer);
	
		sample_rate=32768; sample_time=5; volume=20;
		N=InitAudio(); InitFFT(N);
	
		f = fopen("/usr/local/etc/.gnome-gtune-status","r");
		if(f == NULL)
		{
			interval = DEFAULT_INTERVAL; 
			printf("Could not open /usr/local/etc/.gnome-gtune-status, using default interval\nFor more information, look this up in the help file.\n");
		}
		else
		{
			fscanf(f,"%d",&interval);
			interval *=1.3;
			if(interval == DEFAULT_INTERVAL) interval += 1; /*this is stupid*/
			fclose(f);
		}
		gtk_timeout_add(interval, timer_cb, &context_id);
		
		gtk_widget_hide(dialog);
		return;
	}
	
	pvalue += .1;	
	if(pvalue > 1.0) pvalue = 1.0;
	gtk_progress_bar_update(GTK_PROGRESS_BAR(w), pvalue);
	
}

void
timer_cb(gpointer data)
{ 
	GtkWidget *label, *hscale, *statusbar;
	GtkAdjustment *adj;
	struct note_type note;
	char *output; double cx;

	if(interval == 0) /* ie. still in dialog phase */
	{
		progress_cb();
		return;
	}
		
	/* this function is very simple, it samples the audio, figures out what it is, and displays it through the status bar, slider and label */	
	label = get_widget(app, "C");
	hscale = get_widget(app, "hscale");
	statusbar = get_widget(app, "statusbar");
	
	if (N!=SampleAudio(N)) {
			printf("Sampling error!\n"); quit_cb();
		}
		cx = CalcFFT(N);
		if (!((double) cx <= 0)) note = Note(cx); 
		FlushAudio();
		
	if(!((double) cx <= 0))
	{
		gtk_statusbar_pop( GTK_STATUSBAR(statusbar), (guint) &data );
		gtk_statusbar_push(GTK_STATUSBAR(statusbar),(guint) &data, note.old_style);
		gtk_label_set(GTK_LABEL(label), note.note);
		adj = gtk_range_get_adjustment(GTK_RANGE(hscale));
		gtk_adjustment_set_value( adj, (float) note.precision);
		n_times = 0;
	}
	else
	{
	  /* I do this because I don't want the display to just disappear, leave it on for a bit so it's a little more friendly */
		n_times ++;
		if (n_times > 1) gtk_statusbar_pop(GTK_STATUSBAR(statusbar), (guint) &data);
	}	
}

void 
do_gnome_stuff()
{
	GtkWidget *vbox;
	GtkWidget *C, *hscale, *statusbar, *progress, *l;
	int context_id;
	
	app = gnome_app_new("gTune", "Gnome Tuner");
	
	gtk_widget_realize(app);
	gtk_signal_connect(GTK_OBJECT(app), "delete_event",
					GTK_SIGNAL_FUNC(quit_cb), NULL);
					
	gnome_app_create_menus (GNOME_APP (app), main_menu);

	vbox = gtk_vbox_new(FALSE, 0);
 	gnome_app_set_contents ( GNOME_APP (app), vbox);
 	gtk_widget_show(vbox);

 	C = gtk_label_new ("");
  	gtk_object_set_data (GTK_OBJECT (app), "C", C);
  	gtk_widget_show (C);
  	gtk_box_pack_start (GTK_BOX (vbox), C, TRUE, TRUE, 0);

  	hscale = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (7, 0, 11, 1, 1, 1)));
  	gtk_object_set_data (GTK_OBJECT (app), "hscale", hscale);
  	gtk_widget_show (hscale);
  	gtk_box_pack_start (GTK_BOX (vbox), hscale, TRUE, TRUE, 0);
  	gtk_scale_set_draw_value (GTK_SCALE (hscale), FALSE);

  	statusbar = gtk_statusbar_new ();
  	gtk_object_set_data (GTK_OBJECT (app), "statusbar", statusbar);
  	gtk_widget_show (statusbar);
  	gtk_box_pack_start (GTK_BOX (vbox), statusbar, FALSE, TRUE, 0);
	context_id = gtk_statusbar_get_context_id( GTK_STATUSBAR(statusbar), "Statusbar example");
	timer =	gtk_timeout_add( 850, timer_cb, &context_id);

	/* now lets get this dialog goin */
	dialog = gtk_dialog_new(); /* this one calls for a good old fasion one */	

	l = gtk_label_new("Gathering information...");
	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), l, FALSE, TRUE, 0);
	gtk_widget_show(l);

	progress = gtk_progress_bar_new();
	gtk_object_set_data (GTK_OBJECT (dialog), "progressbar", progress);
	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), progress, FALSE, TRUE, 0);
	gtk_widget_show(progress);

	gtk_widget_show(app);
	gtk_widget_show(dialog);
}

void
quit_cb (GtkWidget *widget, void *data)
{
  CloseAudio();
  gtk_main_quit ();
  return;
}

void
about_cb (GtkWidget *widget, void *data)
{
  GtkWidget *about;
  gchar *authors[] = {
/* Here should be your names */
	"Harold C.L. Baur",
	  "Michael Celli",
          NULL
          };

  about = gnome_about_new ( _("gTune"), NULL,
        		/* copyrigth notice */
                        "Copyright (c) 1999 under the terms of the included LICENSE file",
                        authors,
                    _("This is a translation of gTune for a nice Gnome interface.\n"
                      "All the audio code is by Harold C.L. Baur's original gTune program for the console.\n"
                      "Kirat Singh wrote the original \"tune\" utility from which gTune was derived.\n"
                      "Peter  Valkenburg wrote the FFT routines which are not released under the terms of the included LICENSE file (see the README for more info).\n"),
                        NULL);
  gtk_widget_show (about);

  return;
}
	
