#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#include "audio.h"
#include "calc.h"
#include "note.h"

int debug=0;
int w = 0;

int main(int argc, char **argv) {

	extern int sample_rate, sample_time, volume;
	int c, N, i;
	time_t t;
	struct note_type n;
	FILE *f;

	sample_rate=32768; sample_time=5; volume=20;

	while ((c=getopt(argc,argv,"de:s:t:v:h:w"))!=EOF) switch (c) {
	    case 's':
		sample_rate=atoi(optarg); 
		if (sample_rate<1000) {
		    	printf("%s: %d: invalid sample rate\n",argv[0],sample_rate); 
			exit(1);   /* it should be at least 1 kHz */
		}
		break;
	    case 't':
		sample_time=atoi(optarg); 
		if (sample_time<1) {
		    	printf("%s: %d: invalid sample time\n",argv[0],sample_time); 
			exit(1);   /* it should be at least 0.1 second */
		}
		break;
	    case 'v':
		volume=atoi(optarg); 
		if (volume<1) {
		    	printf("%s: %d: invalid volume\n",argv[0],volume); 
			exit(1);   /* it should be at least 0 */
		}
		break;
	    case 'd': 
		debug=1;
		break;
	    case 'w':
	    	w = 1;
	    	break;
	    case 'h':
		printf("usage: %s -[stvdh]\n", argv[0]);
		printf("-s sample rate (Hz), default 32768\n");
		printf("-t sample time (tens of a second), default 5\n");
		printf("-v mixer volume, default 20\n");
		printf("-d run in debug mode\n");
		printf("-w for use with the interface\n");
		printf("-h shows this help\n"); return 0;
	    case '?':
		break;
	    default :
              printf("?? getopt returned character code 0%o ??",c);

	}

	N=InitAudio(); 
	if(N == -1) exit(1);
	if(InitFFT(N) == -1) exit(1);

/*	
	for testing purposes 
	for (N=220; N<=1100; ++N) Note((double)N); 
	Note(329.6); Note(440.0); Note(587.3); 
	Note(784.0); Note(987.0); Note(1318.5); 
	exit(0);
*/

	i = 0;
	t = time(NULL);
	while (1) {
		if (N!=SampleAudio(N)) {
			printf("Sampling error!\n"); exit(1);
		}
		n = Note(CalcFFT(N)); 
		FlushAudio();
		
		i ++;
		
		if(!w) printf("%s\n",n.old_style);
		else
		{	
			if ((time(NULL) - t) >= 6)
			{
				f = fopen("/usr/local/etc/.gnome-gtune-status","w");
				fprintf(f,"%d",(int) (1000 * ((float) 6 / i)));
				fclose(f);
				CloseAudio();				
				return 0;
			}
		}
			
		if(debug == 1)
		{
			printf("CPU Time so far: %d\n",(int) (clock() / CLOCKS_PER_SEC)); 
			printf("Time so far: %d\n",(int) (time(NULL) - t));
			printf("Ratio of cycles per second: %f\n", 1000 * ((float) (time(NULL) - t) / i));	
		}
		

	} /* conclusion : cycles per second will drop by about 40% as CPU becomes
		more used up */

	CloseAudio();

	return 0;
}


